#!/usr/bin/env python
# _*_ coding: utf-8 _*_
# @Time : 2022/6/1 21:19
# @Author : Rongrui Zhan
# @desc : 本代码未经授权禁止商用
import h5py
import numpy as np
import torch
from typing import Tuple
from torch.utils.data import Dataset

torch.__version__
class ERPDataset(Dataset):
    def __init__(self, path, train=True, sep=561615, device="cpu"):
        with h5py.File(path, 'r') as hf:
            self.features = np.array(hf.get("features"), dtype="float32")
            self.erp_labels = np.array(hf.get("erp_labels"), dtype="long")
        if train:
            self.features = self.features[:sep]
            self.erp_labels = self.erp_labels[:sep]
        else:
            self.features = self.features[sep:]
            self.erp_labels = self.erp_labels[sep:]
        self.erp_labels = torch.from_numpy(self.erp_labels).to(device)
        self.features = torch.from_numpy(self.features.reshape(
            (self.features.shape[0], 1, self.features.shape[1],
             self.features.shape[2])
        )).to(device)

    def __getitem__(self, index) -> Tuple[torch.Tensor, torch.Tensor]:
        return self.features[index], self.erp_labels[index]

    def __len__(self):
        return len(self.erp_labels)



#!/usr/bin/env python
# _*_ coding: utf-8 _*_
# @Time : 2022/6/2 13:08
# @Author : Rongrui Zhan
# @desc : 本代码未经授权禁止商用
import torch
import torch.nn as nn
import torch.nn.functional


class EEGSpatialLSTM(nn.Module):
    def __init__(self, num_classes, input_size, hidden_size, num_layers=2,
                 num_spatial=8, num_times=None, num_channels=8, dropout_rate=0.5, dependent=False):
        super().__init__()
        self.input_size = input_size
        self.hidden_size = hidden_size

        self.len_seq = None if num_times is None else num_times // input_size
        self.num_channels = num_channels
        self.num_layers = num_layers
        if dependent:
            self.lstm = nn.ModuleList([
                nn.LSTM(input_size, hidden_size, num_layers, batch_first=True)
                for _ in range(num_channels)
            ])
        else:
            self.lstm = nn.LSTM(input_size, hidden_size, num_layers, batch_first=True)

        self.block = nn.Sequential(
            nn.Conv2d(1, num_spatial, (num_channels, 1)),
            nn.BatchNorm2d(num_spatial),
            nn.ELU(),
            nn.AvgPool2d((1, 4)),
            nn.Dropout(dropout_rate)
        )
        self.cls = nn.Sequential(
            nn.Linear(num_spatial * hidden_size // 4, num_classes),
            nn.Softmax(1)
        )

    def forward(self, x):
        # shape of input: Nx1xTxC
        x = x.permute(dims=(0, 1, 3, 2))
        # shape of x: Nx1xCxT
        if self.len_seq is None:
            x = x.view(x.shape[0], self.num_channels, x.shape[-1] // self.input_size, self.input_size)
        else:
            x = x.view(x.shape[0], self.num_channels, self.len_seq, self.input_size)
        # shape of x: NxCxLxI
        if isinstance(self.lstm, nn.ModuleList):
            x = torch.cat([
                lstm(x[:, i])[0][:, -1].unsqueeze(1) for i, lstm in enumerate(self.lstm)
                # shape of output: NxLxH -> Nx1xH
            ], dim=1)
        else:
            x = torch.cat([
                self.lstm(x[:, i])[0][:, -1].unsqueeze(1) for i in range(self.num_channels)
            ], dim=1)
        # shape of x: NxCxH
        x = x.unsqueeze(1)
        x = self.block(x)
        x = torch.flatten(x, start_dim=1)
        return self.cls(x)


if __name__ == '__main__':
    input = torch.zeros(10, 1, 128, 8)
    print(EEGSpatialLSTM(2, input_size=16, hidden_size=16)(input)[0].shape)
    # print(nn.LSTM(input_size=8, hidden_size=16)(input.squeeze(1))[1][0].shape)
    # print(nn.LSTM(input_size=8, hidden_size=16)(input.squeeze(1))[1][1].shape)
    print(torch.arange(12).view(4, 3))



#!/usr/bin/env python
# _*_ coding: utf-8 _*_
# @Time : 2022/6/5 14:30
# @Author : Rongrui Zhan
# @desc : 本代码未经授权禁止商用
import eeg_networks
from setuptools import setup, find_packages

with open("README.md", "r", encoding='UTF-8') as file:
    long_description = file.read()

setup(
    name="EEGNetworks",
    version=eeg_networks.__version__,
    author="六个骨头",
    author_email="2742392377@qq.com",
    description="""
    基于PyTorch的各种用于EEG分类任务的简单网络的实现。
    """,
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/zrr1999/eeg-networks-pytorch",
    # 你要安装的包，通过 setuptools.find_packages 找到当前目录下有哪些包
    packages=find_packages(exclude=["tests", "_*"]),
    package_data={
        '': ['*.txt'],
        # 包含demo包data文件夹中的 *.dat文件
        'demo': ['data/*.dat'],
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU Lesser General Public License v3 (LGPLv3)",
        "Operating System :: OS Independent",
    ],
    include_package_data=True,
    platforms="any",
    install_requires=["pytorch", "tqdm", "numpy", "h5py"],
    # extras_require={
    #     'server': ['fastapi', 'pydantic', 'uvicorn>=0.3.0'],
    #     'parse': ['pyparsing'],
    #     'command': ['typer'],
    # },
    entry_points={
        # 'console_scripts': [
        #     'bonetex=bonetex:main',
        # ]
    }
)




#!/usr/bin/env python
# _*_ coding: utf-8 _*_
# @Time : 2022/6/5 16:01
# @Author : Rongrui Zhan
# @desc : 本代码未经授权禁止商用
__version__ = "0.1.0"
from .nets import EEGInception, EEGSpatialLSTM
from .dataset import ERPDataset



#!/usr/bin/env python
# _*_ coding: utf-8 _*_
# @Time : 2022/6/1 18:55
# @Author : Rongrui Zhan
# @desc : 本代码未经授权禁止商用
import torch
import torch.nn as nn
import torch.nn.functional


class CustomPad(nn.Module):
    def __init__(self, padding):
        super().__init__()
        self.padding = padding

    def forward(self, x):
        return nn.functional.pad(x, self.padding)


class DepthWiseConv2d(nn.Module):
    def __init__(self, channel, kernel_size, depth_multiplier, bias=False):
        super().__init__()
        self.nets = nn.ModuleList([
            nn.Conv2d(channel, channel, kernel_size, bias=bias, groups=channel)
            for _ in range(depth_multiplier)
        ])

    def forward(self, x):
        output = torch.cat([net(x) for net in self.nets], 1)
        return output


class EEGInception(nn.Module):
    def __init__(self, num_classes, fs=1282, num_channels=8, filters_per_branch=8,
                 scales_time=(500, 250, 125), dropout_rate=0.25,
                 activation=nn.ELU(inplace=True)):
        super().__init__()
        scales_samples = [int(s * fs / 1000) for s in scales_time]
        # ========================== BLOCK 1: INCEPTION ========================== #
        self.inception1 = nn.ModuleList([
            nn.Sequential(
                nn.Conv2d(
                    1, filters_per_branch, (scales_sample, 1),
                    padding="same"
                    # padding=((scales_sample - 1) // 2, 0)
                ) if torch.__version__ >= "1.9" else nn.Sequential(
                    CustomPad((0, 0,scales_sample // 2 - 1, scales_sample // 2, )),
                    nn.Conv2d(
                        1, filters_per_branch, (scales_sample, 1)
                    )
                ),
                nn.BatchNorm2d(filters_per_branch),
                activation,
                nn.Dropout(dropout_rate),
                DepthWiseConv2d(8, (1, num_channels), 2),
                nn.BatchNorm2d(filters_per_branch * 2),
                activation,
                nn.Dropout(dropout_rate),
            ) for scales_sample in scales_samples
        ])
        self.avg_pool1 = nn.AvgPool2d((4, 1))

        # ========================== BLOCK 2: INCEPTION ========================== #
        self.inception2 = nn.ModuleList([
            nn.Sequential(
                nn.Conv2d(
                    len(scales_samples) * 2 * filters_per_branch,
                    filters_per_branch, (scales_sample // 4, 1),
                    bias=False,
                    padding="same"
                    # padding=((scales_sample // 4 - 1) // 2, 0)
                ) if torch.__version__ >= "1.9" else nn.Sequential(
                    CustomPad((0, 0, scales_sample // 8 - 1, scales_sample // 8, )),
                    nn.Conv2d(
                        len(scales_samples) * 2 * filters_per_branch,
                        filters_per_branch, (scales_sample // 4, 1),
                        bias=False
                    )
                ),
                nn.BatchNorm2d(filters_per_branch),
                activation,
                nn.Dropout(dropout_rate),
            ) for scales_sample in scales_samples
        ])

        self.avg_pool2 = nn.AvgPool2d((2, 1))

        # ============================ BLOCK 3: OUTPUT =========================== #
        self.output = nn.Sequential(
            nn.Conv2d(
                24, filters_per_branch * len(scales_samples) // 2, (8, 1),
                bias=False, padding='same'
            ) if torch.__version__ >= "1.9" else nn.Sequential(
                CustomPad((0, 0, 4, 3)),
                nn.Conv2d(
                    24, filters_per_branch * len(scales_samples) // 2, (8, 1),
                    bias=False
                )
            ),
            nn.BatchNorm2d(filters_per_branch * len(scales_samples) // 2),
            activation,
            nn.AvgPool2d((2, 1)),
            nn.Dropout(dropout_rate),

            nn.Conv2d(
                12, filters_per_branch * len(scales_samples) // 4, (4, 1),
                bias=False, padding='same'
            ) if torch.__version__ >= "1.9" else nn.Sequential(
                CustomPad((0, 0, 2, 1)),
                nn.Conv2d(
                    12, filters_per_branch * len(scales_samples) // 4, (4, 1),
                    bias=False
                )
            ),
            nn.BatchNorm2d(filters_per_branch * len(scales_samples) // 4),
            activation,
            nn.AvgPool2d((2, 1)),
            nn.Dropout(dropout_rate),
        )
        self.cls = nn.Sequential(
            nn.Linear(4 * 1 * 6, num_classes),
            nn.Softmax(1)
        )

    def forward(self, x):
        x = torch.cat([net(x) for net in self.inception1], dim=1)
        x = self.avg_pool1(x)
        x = torch.cat([net(x) for net in self.inception2], dim=1)
        x = self.avg_pool2(x)
        x = self.output(x)
        x = torch.flatten(x, start_dim=1)
        return self.cls(x)


if __name__ == '__main__':
    print(nn.functional.pad(torch.zeros(10, 1, 128, 8), (37, 37, 0, 0)).shape)
    # print(EEGInception()(torch.zeros(10, 1, 128, 8)).shape)
    # print(DepthWiseConv2d(3, (3, 3), 2)(torch.zeros(10, 3, 512, 512)).shape)
    # print(nn.AvgPool2d((2, 1))(torch.zeros(10, 3, 512, 512)).shape)
